@@ -261,6 +261,8 @@ static const struct client_surface_funcs x11drv_client_surface_funcs;
 struct x11drv_client_surface
 {
     struct client_surface client;
+    XWindowChanges changes;
+    Colormap colormap;
     Window window;
     RECT rect;
 
@@ -280,6 +282,7 @@ static void x11drv_client_surface_destroy( struct client_surface *client )
 
     TRACE( "%s\n", debugstr_client_surface( client ) );
 
+    if (surface->colormap != default_colormap) XFreeColormap( gdi_display, surface->colormap );
     if (surface->window) destroy_client_window( hwnd, surface->window );
     if (surface->hdc_dst) NtGdiDeleteObjectApp( surface->hdc_dst );
     if (surface->hdc_src) NtGdiDeleteObjectApp( surface->hdc_src );
@@ -301,18 +304,40 @@ static void x11drv_client_surface_detach( struct client_surface *client )
     }
 }
 
-static void client_surface_update_size( HWND hwnd, struct x11drv_client_surface *surface )
+static void client_surface_update_geometry( HWND hwnd, struct x11drv_client_surface *surface )
 {
-    XWindowChanges changes;
+    HWND toplevel = NtUserGetAncestor( hwnd, GA_ROOT );
+    XWindowChanges changes = surface->changes;
+    struct x11drv_win_data *data;
+    int mask = 0;
     RECT rect;
 
     if (!NtUserGetClientRect( hwnd, &rect, NtUserGetDpiForWindow( hwnd ) )) return;
-    if (EqualRect( &surface->rect, &rect )) return;
+    NtUserMapWindowPoints( hwnd, toplevel, (POINT *)&rect, 2, NtUserGetDpiForWindow( hwnd ) );
+    if ((data = get_win_data( toplevel )))
+    {
+        OffsetRect( &rect, data->rects.client.left - data->rects.visible.left,
+                    data->rects.client.top - data->rects.visible.top );
+        release_win_data( data );
+    }
 
-    changes.width  = min( max( 1, rect.right ), 65535 );
-    changes.height = min( max( 1, rect.bottom ), 65535 );
-    XConfigureWindow( gdi_display, surface->window, CWWidth | CWHeight, &changes );
+    changes.x = rect.left;
+    changes.y = rect.top;
+    changes.width  = min( max( 1, rect.right - rect.left ), 65535 );
+    changes.height = min( max( 1, rect.bottom - rect.top ), 65535 );
+    OffsetRect( &rect, -rect.left, -rect.top );
     surface->rect = rect;
+
+    if (changes.x != surface->changes.x) mask |= CWX;
+    if (changes.y != surface->changes.y) mask |= CWY;
+    if (changes.width != surface->changes.width) mask |= CWWidth;
+    if (changes.height != surface->changes.height) mask |= CWHeight;
+    if (!mask) return;
+
+    surface->changes = changes;
+    TRACE( "client window %p/%lx, requesting position %d,%d size %d,%d mask %#x\n", hwnd,
+           surface->window, changes.x, changes.y, changes.width, changes.height, mask );
+    XConfigureWindow( gdi_display, surface->window, mask, &changes );
 }
 
 static void client_surface_update_offscreen( HWND hwnd, struct x11drv_client_surface *surface )
@@ -350,7 +375,7 @@ static void client_surface_update_offscreen( HWND hwnd, struct x11drv_client_sur
     }
     else
     {
-        static const WCHAR displayW[] = {'D','I','S','P','L','A','Y'};
+        static const WCHAR displayW[] = {'D','I','S','P','L','A','Y', 0};
         UNICODE_STRING device_str = RTL_CONSTANT_STRING(displayW);
         surface->hdc_dst = NtGdiOpenDCW( &device_str, NULL, NULL, 0, TRUE, NULL, NULL, NULL );
         surface->hdc_src = NtGdiOpenDCW( &device_str, NULL, NULL, 0, TRUE, NULL, NULL, NULL );
@@ -375,7 +400,7 @@ static void x11drv_client_surface_update( struct client_surface *client )
 
     TRACE( "%s\n", debugstr_client_surface( client ) );
 
-    client_surface_update_size( hwnd, surface );
+    client_surface_update_geometry( hwnd, surface );
     client_surface_update_offscreen( hwnd, surface );
 }
 
@@ -390,7 +415,7 @@ static void X11DRV_client_surface_present( struct client_surface *client, HDC hd
 
     TRACE( "%s\n", debugstr_client_surface( client ) );
 
-    client_surface_update_size( hwnd, surface );
+    client_surface_update_geometry( hwnd, surface );
     client_surface_update_offscreen( hwnd, surface );
 
     if (!hdc) return;
@@ -413,6 +438,7 @@ static void X11DRV_client_surface_present( struct client_surface *client, HDC hd
 
     NtGdiStretchBlt( surface->hdc_dst, 0, 0, rect_dst.right - rect_dst.left, rect_dst.bottom - rect_dst.top,
                      surface->hdc_src, 0, 0, surface->rect.right, surface->rect.bottom, SRCCOPY, 0 );
+    XFlush( gdi_display );
 
 done:
     if (region) NtGdiDeleteObjectApp( region );
@@ -426,25 +452,37 @@ static const struct client_surface_funcs x11drv_client_surface_funcs =
     .present = X11DRV_client_surface_present,
 };
 
-Window x11drv_client_surface_create( HWND hwnd, const XVisualInfo *visual, Colormap colormap, struct client_surface **client )
+static int visual_class_alloc( int class )
+{
+    return class == PseudoColor || class == GrayScale || class == DirectColor ? AllocAll : AllocNone;
+}
+
+Window x11drv_client_surface_create( HWND hwnd, int format, struct client_surface **client )
 {
     struct x11drv_client_surface *surface;
+    XVisualInfo visual = default_visual;
+    Colormap colormap;
 
-    if (!(surface = client_surface_create( sizeof(*surface), &x11drv_client_surface_funcs, hwnd ))) return None;
-    if (!(surface->window = create_client_window( hwnd, visual, colormap )))
-    {
-        client_surface_release( &surface->client );
-        return None;
-    }
-    if (!NtUserGetClientRect( hwnd, &surface->rect, NtUserGetDpiForWindow( hwnd ) ))
-    {
-        client_surface_release( &surface->client );
-        return None;
-    }
+    if (format && !visual_from_pixel_format( format, &visual )) return None;
+
+    if (visual.visualid == default_visual.visualid) colormap = default_colormap;
+    else colormap = XCreateColormap( gdi_display, get_dummy_parent(), visual.visual, visual_class_alloc( visual.class ) );
+    if (!colormap) return None;
+
+    if (!(surface = client_surface_create( sizeof(*surface), &x11drv_client_surface_funcs, hwnd ))) goto failed;
+    surface->colormap = colormap;
+
+    if (!NtUserGetClientRect( hwnd, &surface->rect, NtUserGetDpiForWindow( hwnd ) )) goto failed;
+    if (!(surface->window = create_client_window( hwnd, surface->rect, &visual, colormap ))) goto failed;
 
     TRACE( "Created %s for client window %lx\n", debugstr_client_surface( &surface->client ), surface->window );
     *client = &surface->client;
     return surface->window;
+
+failed:
+    if (surface) client_surface_release( &surface->client );
+    else if (colormap != default_colormap) XFreeColormap( gdi_display, colormap );
+    return None;
 }
 
 /**********************************************************************
@@ -640,7 +678,7 @@ static const struct user_driver_funcs x11drv_funcs =
     .pActivateWindow = X11DRV_ActivateWindow,
     .pSetLayeredWindowAttributes = X11DRV_SetLayeredWindowAttributes,
     .pSetParent = X11DRV_SetParent,
-    .pSetWindowIcon = X11DRV_SetWindowIcon,
+    .pSetWindowIcons = X11DRV_SetWindowIcons,
     .pSetWindowRgn = X11DRV_SetWindowRgn,
     .pSetWindowStyle = X11DRV_SetWindowStyle,
     .pSetWindowText = X11DRV_SetWindowText,
@@ -657,6 +695,7 @@ static const struct user_driver_funcs x11drv_funcs =
     .pMoveWindowBits = X11DRV_MoveWindowBits,
     .pWindowPosChanged = X11DRV_WindowPosChanged,
     .pSystemParametersInfo = X11DRV_SystemParametersInfo,
+    .pWintabProc = X11DRV_WintabProc,
     .pVulkanInit = X11DRV_VulkanInit,
     .pOpenGLInit = X11DRV_OpenGLInit,
     .pThreadDetach = X11DRV_ThreadDetach,
