@@ -210,7 +210,7 @@ static Cursor get_empty_cursor(void)
 /***********************************************************************
  *		set_window_cursor
  */
-void set_window_cursor( Window window, HCURSOR handle )
+static void set_window_cursor( Window window, HCURSOR handle )
 {
     Cursor cursor, prev;
 
@@ -1538,8 +1538,8 @@ BOOL X11DRV_GetCursorPos(LPPOINT pos)
  */
 BOOL X11DRV_ClipCursor( const RECT *clip, BOOL reset )
 {
-    if (!reset && clip && grab_clipping_window( clip )) return TRUE;
-    ungrab_clipping_window();
+    if (reset || !clip || !grab_clipping_window( clip )) ungrab_clipping_window();
+    XFlush( x11drv_thread_data()->display );
     return TRUE;
 }
 
